import { h, Component } from 'preact';
import { route } from 'preact-router';
import makeArvadosRequest from 'make-arvados-request';
import WBTable from 'wb-table';
import WBPagination from 'wb-pagination';
import urlForObject from 'url-for-object';
import arvadosTypeName from 'arvados-type-name';
import arvadosObjectName from 'arvados-object-name';

class WBCollectionListing extends Component {

  constructor(...args) {
    super(...args);
    this.state.rows = [];
    this.state.numPages = 0;
  }

  componentDidMount() {
    this.fetchItems();
  }

  prepareRows(items, ownerLookup) {
    return items.map(item => [
      (<div>
        <div>
          <a href={ urlForObject(item) }>{ item['name'] }</a>
        </div>
        <div>{ item['uuid'] }</div>
      </div>),
      item['description'],
      (<div>
        <div>
          { ownerLookup[item.owner_uuid] ? (
            <a href={ urlForObject(ownerLookup[item.owner_uuid]) }>
              { arvadosObjectName(ownerLookup[item.owner_uuid]) }
            </a>
          ) : 'Not Found' }
        </div>
        <div>{ item.owner_uuid }</div>
      </div>),
      item['file_count'],
      filesize(item['file_size_total'])
    ]);
  }

  fetchItems() {
    let { arvHost, arvToken } = this.props.app.state;

    let { activePage, itemsPerPage, ownerUuid } = this.props;

    let filters = [];

    if (ownerUuid)
      filters.push([ 'owner_uuid', '=', ownerUuid ]);

    let prom = makeArvadosRequest(arvHost, arvToken,
      '/arvados/v1/collections?filters=' + encodeURIComponent(JSON.stringify(filters)) +
      '&limit=' + encodeURIComponent(itemsPerPage) +
      '&offset=' + encodeURIComponent(itemsPerPage * activePage));

    let collections;
    let numPages

    prom = prom.then(xhr => {
      collections = xhr.response['items'];
      numPages = Math.ceil(xhr.response['items_available'] / xhr.response['limit']);

      let owners = {};
      collections.map(c => {
        let typeName = arvadosTypeName(c.owner_uuid);
        if (!(typeName in owners))
          owners[typeName] = [];
        owners[typeName].push(c.owner_uuid);
      });

      let lookup = {};
      let prom_1 = new Promise(accept => accept());
      for (let typeName in owners) {
        let filters_1 = [
          ['uuid', 'in', owners[typeName]]
        ];
        prom_1 = prom_1.then(() => makeArvadosRequest(arvHost, arvToken,
          '/arvados/v1/' + typeName + 's?filters=' +
          encodeURIComponent(JSON.stringify(filters_1))));
        prom_1 = prom_1.then(xhr => xhr.response.items.map(item => (
          lookup[item.uuid] = item)));
      }

      prom_1 = prom_1.then(() => lookup);

      return prom_1;
    });

    //let ownerLookup = {};
    //prom = prom.then(lookup => (ownerLookup = lookup));

    prom = prom.then(ownerLookup =>
      this.setState({
        'numPages': numPages,
        'rows': this.prepareRows(collections, ownerLookup)
      }));
  }

  componentWillReceiveProps(nextProps, nextState) {
    this.props = nextProps;
    this.fetchItems();
  }

  render({ app, ownerUuid, activePage, getPageUrl }, { rows, numPages }) {
    return (
      <div>
        <WBTable columns={ [ 'Name', 'Description', 'Owner', 'File Count', 'Total Size' ] }
          rows={ rows } />

        <WBPagination numPages={ numPages }
          activePage={ activePage }
          getPageUrl={ getPageUrl } />
      </div>
    );
  }
}

WBCollectionListing.defaultProps = {
  'itemsPerPage': 100,
  'ownerUuid': null
};

export default WBCollectionListing;
